package com.lantone.security.facade;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.additional.query.impl.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.service.additional.update.impl.LambdaUpdateChainWrapper;
import com.lantone.common.dto.DeptListDTO;
import com.lantone.common.dto.ViewRegionDTO;
import com.lantone.common.exception.ApiException;
import com.lantone.common.util.ListUtil;
import com.lantone.common.util.StringUtil;
import com.lantone.common.util.SysUserUtils;
import com.lantone.common.vo.CheckSolveRegionVO;
import com.lantone.common.vo.GetDeptListVO;
import com.lantone.common.vo.SaveRegionVO;
import com.lantone.common.vo.GetRegionVO;
import com.lantone.dblayermbg.entity.Dept;
import com.lantone.dblayermbg.entity.Region;
import com.lantone.dblayermbg.entity.RegionDept;
import com.lantone.dblayermbg.facade.DeptFacade;
import com.lantone.dblayermbg.facade.RegionFacade;
import com.lantone.dblayermbg.service.RegionDeptService;
import com.lantone.common.enums.IsDeleteEnum;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 病区表 服务实现类Facade
 * </p>
 */
@Component
public class RegionManagementFacade {
    @Autowired
    @Qualifier("regionDeptServiceImpl")
    private RegionDeptService regionDeptService;
    @Autowired
    private RegionFacade regionFacade;
    @Autowired
    private DeptFacade deptFacade;

    /**
     * @Description: 新增/修改病区
     * @Param: [saveRegionVo]
     * @return: java.lang.Object
     * @Author: cy
     * @Date: 2021/7/30
     */
    public Object saveRegion(SaveRegionVO saveRegionVo) {
        saveRegionVo.setHospitalId(SysUserUtils.getCurrentHospitalId());
        saveRegionVo.setUserId(SysUserUtils.getCurrentPrincipleId());
        Boolean flag = false;
        //新增-校验重复数据
        getCheckRegion(saveRegionVo);
        Region sysRegion = new Region();
        BeanUtils.copyProperties(saveRegionVo, sysRegion);
        List<RegionDept> sysRegionDepts = new ArrayList<>();
        if (null == saveRegionVo.getId()) {
            flag = addRegion(saveRegionVo, sysRegion, sysRegionDepts);
        } else if (null != saveRegionVo.getId()) {
            flag = updateRegion(saveRegionVo, sysRegion, sysRegionDepts);
        }
        return flag;
    }

    /**
     * @Description: 查看病区管理
     * @Param: [getRegionVO]
     * @return: IPage<com.lantone.common.dto.ViewRegionDTO>
     * @Author: cy
     * @Date: 2021/7/30
     */
    public IPage<ViewRegionDTO> getRegion(GetRegionVO getRegionVO) {
        getRegionVO.setHospitalId(SysUserUtils.getCurrentHospitalId());
        IPage<ViewRegionDTO> page = regionFacade.getWard(getRegionVO);
        List<ViewRegionDTO> wardList = page.getRecords();
        if (ListUtil.isNotEmpty(wardList) && StringUtil.isNotEmpty(getRegionVO.getDeptName())) {
            IPage pageOne = new Page();
            List<ViewRegionDTO> viewRegionDTOS = new ArrayList<>();
            wardList.forEach(obj -> {
                if (StringUtil.isNotEmpty(obj.getDeptName()) && obj.getDeptName().contains(getRegionVO.getDeptName())) {
                    viewRegionDTOS.add(obj);
                }
            });
            pageOne.setRecords(viewRegionDTOS);
            return pageOne;
        }
        return page;
    }

    /**
     * @Description: 删除病区管理
     * @Param: [id]
     * @return: java.lang.Boolean
     * @Author: cy
     * @Date: 2021/7/30
     */
    public Boolean deleteRegion(Integer id) {
        Long hospitalId = SysUserUtils.getCurrentHospitalId();
        regionFacade.lambdaUpdate()
                .eq(Region::getId,id)
                .eq(Region::getHospitalId, hospitalId).remove();
        boolean flag = regionDeptService.lambdaUpdate()
                .eq(RegionDept::getRegionId, id)
                .eq(RegionDept::getHospitalId, hospitalId).remove();
        return flag;
    }

    /**
     * @Description: 新增-校验重复数据
     * @Param: [saveRegionVo]
     * @return: void
     * @Author: cy
     * @Date: 2021/7/30
     */
    public void getCheckRegion(SaveRegionVO saveRegionVo) {
        if (null == saveRegionVo.getId()) {
            Region region = regionFacade.lambdaQuery()
                    .eq(Region::getIsDeleted, IsDeleteEnum.N.getKey())
                    .eq(Region::getName, saveRegionVo.getName())
                    .eq(Region::getHospitalId, saveRegionVo.getHospitalId())
                    .one();
            if(null != region && null != region.getId()){
                throw new ApiException("当前组织和病区已重复");
            }
        }
    }

    /**
     * @Description: 新增病区
     * @Param: [saveRegionVo, sysRegion, sysRegionDepts]
     * @return: java.lang.Boolean
     * @Author: cy
     * @Date: 2021/7/30
     */
    public Boolean addRegion(SaveRegionVO saveRegionVo, Region sysRegion, List<RegionDept> sysRegionDepts) {
        sysRegion.setGmtCreate(new Date());
        sysRegion.setCreator(saveRegionVo.getUserId().toString());
        regionFacade.save(sysRegion);
        if (ListUtil.isNotEmpty(saveRegionVo.getDeptIds())) {
            saveRegionVo.getDeptIds().forEach(obj -> {
                RegionDept sysRegionDept = new RegionDept();
                BeanUtils.copyProperties(saveRegionVo, sysRegionDept);
                sysRegionDept.setDeptId(obj);
                sysRegionDept.setRegionId(sysRegion.getId());
                sysRegionDept.setGmtCreate(new Date());
                sysRegionDept.setCreator(saveRegionVo.getUserId().toString());
                sysRegionDepts.add(sysRegionDept);
            });
            return regionDeptService.saveBatch(sysRegionDepts);
        }
        return true;
    }

    /**
     * @Description: 修改病区
     * @Param: [saveRegionVo, sysRegion, sysRegionDepts]
     * @return: java.lang.Boolean
     * @Author: cy
     * @Date: 2021/7/30
     */
    public Boolean updateRegion(SaveRegionVO saveRegionVo, Region sysRegion, List<RegionDept> sysRegionDepts) {
        sysRegion.setGmtModified(new Date());
        sysRegion.setModifier(saveRegionVo.getUserId().toString());
        regionFacade.updateById(sysRegion);
        if (ListUtil.isNotEmpty(saveRegionVo.getDeptIds())) {
            Region region = regionFacade.lambdaQuery()
                    .eq(Region::getId, saveRegionVo.getId())
                    .eq(Region::getIsDeleted, IsDeleteEnum.N.getKey()).one();
            regionDeptService.lambdaUpdate()
                    .eq(RegionDept::getHospitalId, saveRegionVo.getHospitalId())
                    .eq(RegionDept::getIsDeleted, IsDeleteEnum.N.getKey())
                    .eq(RegionDept::getRegionId, saveRegionVo.getId()).remove();
            saveRegionVo.getDeptIds().forEach(obj -> {
                RegionDept sysRegionDeptEntry = new RegionDept();
                BeanUtils.copyProperties(saveRegionVo, sysRegionDeptEntry);
                sysRegionDeptEntry.setId(null);
                sysRegionDeptEntry.setDeptId(obj);
                sysRegionDeptEntry.setRegionId(saveRegionVo.getId());
                if (null != region) {
                    sysRegionDeptEntry.setGmtCreate(region.getGmtCreate());
                    sysRegionDeptEntry.setCreator(region.getCreator());
                }
                sysRegionDeptEntry.setGmtModified(new Date());
                sysRegionDeptEntry.setModifier(saveRegionVo.getUserId().toString());
                sysRegionDepts.add(sysRegionDeptEntry);
            });
            return regionDeptService.saveBatch(sysRegionDepts);
        }
        return true;
    }

    /**
     * @Description: 校验/禁用-启用病区管理
     * @Param: [checkSolveRegionVO]
     * @return: java.lang.Object
     * @Author: cy
     * @Date: 2021/7/30
     */
    public Object checkSolveRegion(CheckSolveRegionVO checkSolveRegionVO) {
        checkSolveRegionVO.setHospitalId(SysUserUtils.getCurrentHospitalId());
        checkSolveRegionVO.setUserId(SysUserUtils.getCurrentPrincipleId());
        //禁用校验
        if (null != checkSolveRegionVO.getId() && 0 == checkSolveRegionVO.getMark()) {
            List<Long> collectList = regionDeptService.lambdaQuery()
                    .eq(RegionDept::getIsDeleted, IsDeleteEnum.N.getKey())
                    .eq(RegionDept::getRegionId,checkSolveRegionVO.getId())
                    .eq(RegionDept::getHospitalId,checkSolveRegionVO.getHospitalId())
                    .list().stream().map(obj -> obj.getDeptId()).collect(Collectors.toList());
            if (ListUtil.isNotEmpty(collectList)) {
                //当前病区存在与之关联的科室
                return 1;
            }
            return 0;
        }

        //禁用-启用病区处理
        if (null != checkSolveRegionVO.getId() && (1 == checkSolveRegionVO.getMark() || 2 == checkSolveRegionVO.getMark())) {
            LambdaUpdateChainWrapper<Region> regionLambdaUpdate= regionFacade.lambdaUpdate();
            regionLambdaUpdate.eq(Region::getIsDeleted, IsDeleteEnum.N.getKey());
            regionLambdaUpdate.eq(Region::getId,checkSolveRegionVO.getId());
            regionLambdaUpdate.eq(Region::getHospitalId,checkSolveRegionVO.getHospitalId());
            if(1 == checkSolveRegionVO.getMark()){
                regionLambdaUpdate.set(Region::getStatus,0).update();
            }
            if(2 == checkSolveRegionVO.getMark()){
                regionLambdaUpdate.set(Region::getStatus,1).update();
            }
            regionLambdaUpdate.set(Region::getModifier,checkSolveRegionVO.getUserId().toString()).update();
            regionLambdaUpdate.set(Region::getGmtModified,new Date()).update();
        }
        return true;
    }

    /**
     * @Description:科室id，name返回
     * @Param: [hospitalId]
     * @return: java.util.List<com.lantone.dblayermbg.entity.Dept>
     * @Author: cy
     * @Date: 2021/8/2
     */
    public  List<DeptListDTO> getDept(GetDeptListVO getDeptListVO){
        getDeptListVO.setHospitalId(SysUserUtils.getCurrentHospitalId());
        List<DeptListDTO> deptListDTOS = new ArrayList<>();
        LambdaQueryChainWrapper<Dept> lambdaQueryChainWrapper = deptFacade.lambdaQuery();
        lambdaQueryChainWrapper.eq(Dept::getHospitalId,getDeptListVO.getHospitalId());
        lambdaQueryChainWrapper.eq(Dept::getIsDeleted,IsDeleteEnum.N.getKey());
        if(StringUtil.isNotEmpty(getDeptListVO.getDeptName())){
            lambdaQueryChainWrapper.like(Dept::getName,getDeptListVO.getDeptName());
        }
        lambdaQueryChainWrapper.select(Dept::getId, Dept::getName).list().forEach(
                obj->{
                    DeptListDTO deptListDTO = new DeptListDTO();
                    BeanUtils.copyProperties(obj,deptListDTO);
                    deptListDTOS.add(deptListDTO);
                }
        );
        return deptListDTOS;
    }
}